import { Rule, RuleOptions } from './Rule';
import { RawRuleFrom } from './RawRule';
import { CanParameters, Abilities, Normalize, SubjectType } from './types';
import { LinkedItem } from './structures/LinkedItem';
export interface RuleIndexOptions<A extends Abilities, C> extends Partial<RuleOptions<A, C>> {
    detectSubjectType?(subject?: Normalize<A>[1]): string;
}
declare const $abilities: unique symbol;
declare const $conditions: unique symbol;
interface WithGenerics {
    [$abilities]: any;
    [$conditions]: any;
}
export declare type Public<T extends WithGenerics> = {
    [K in keyof T]: T[K];
};
export declare type Generics<T extends WithGenerics> = {
    abilities: T[typeof $abilities];
    conditions: T[typeof $conditions];
};
export declare type RuleOf<T extends WithGenerics> = Rule<Generics<T>['abilities'], Generics<T>['conditions']>;
export declare type RawRuleOf<T extends WithGenerics> = RawRuleFrom<Generics<T>['abilities'], Generics<T>['conditions']>;
export declare type RuleIndexOptionsOf<T extends WithGenerics> = RuleIndexOptions<Generics<T>['abilities'], Generics<T>['conditions']>;
interface AbilityEvent<T extends WithGenerics> {
    target: T;
    /** @deprecated use "target" property instead */
    ability: T;
}
export interface UpdateEvent<T extends WithGenerics> extends AbilityEvent<T> {
    rules: RawRuleOf<T>[];
}
export declare type EventHandler<Event> = (event: Event) => void;
export declare type Events<T extends WithGenerics, K extends keyof EventsMap<T> = keyof EventsMap<T>> = Map<K, LinkedItem<EventHandler<EventsMap<T>[K]>> | null>;
interface EventsMap<T extends WithGenerics> {
    update: UpdateEvent<T>;
    updated: UpdateEvent<T>;
}
export declare type Unsubscribe = () => void;
export declare class RuleIndex<A extends Abilities, Conditions> {
    private _hasPerFieldRules;
    private _events;
    private _indexedRules;
    private _rules;
    private readonly _ruleOptions;
    readonly detectSubjectType: Exclude<RuleIndexOptions<A, Conditions>['detectSubjectType'], undefined>;
    readonly [$abilities]: A;
    readonly [$conditions]: Conditions;
    constructor(rules?: RawRuleFrom<A, Conditions>[], options?: RuleIndexOptions<A, Conditions>);
    get rules(): (import("./types").ToAbilityTypes<A> extends import("./types").AbilityTupleType<string, SubjectType> ? import("./RawRule").SubjectRawRule<(import("./types").AbilityTupleType<string, SubjectType> & import("./types").ToAbilityTypes<A>)[0], (import("./types").AbilityTupleType<string, SubjectType> & import("./types").ToAbilityTypes<A>)[1], Conditions> : import("./RawRule").ClaimRawRule<Extract<import("./types").ToAbilityTypes<A>, string>>)[];
    update(rules: RawRuleFrom<A, Conditions>[]): Public<this>;
    private _buildIndexFor;
    possibleRulesFor(...args: CanParameters<A, false>): Rule<A, Conditions>[];
    rulesFor(...args: CanParameters<A>): Rule<A, Conditions>[];
    on<T extends keyof EventsMap<this>>(event: T, handler: EventHandler<EventsMap<Public<this>>[T]>): Unsubscribe;
    private _emit;
}
export {};
